@@ -17,8 +17,9 @@ def open(filename, mode="r", iline = 189,
 
     For reading, the access mode "r" is preferred. All write operations will
     raise an exception. For writing, the mode "r+" is preferred (as "rw" would
-    truncate the file). The modes used are standard C file modes; please refer
-    to that documentation for a complete reference.
+    truncate the file). Any mode with 'w' will raise an error. The modes used
+    are standard C file modes; please refer to that documentation for a
+    complete reference.
 
     Open should be used together with python's `with` statement. Please refer
     to the examples. When the `with` statement is used the file will
@@ -66,20 +67,19 @@ def open(filename, mode="r", iline = 189,
             ...
     :rtype: segyio.SegyFile
     """
+
+    if 'w' in mode:
+        problem = 'w in mode would truncate the file'
+        solution = 'use r+ to open in read-write'
+        raise ValueError(', '.join((problem, solution)))
+
     f = segyio.SegyFile(filename, mode, iline, xline)
+    metrics = f.xfd.metrics()
 
     try:
-        metrics = segyio._segyio.init_metrics(f.xfd, f.bin.buf)
-
-        f._tr0 = metrics['trace0']
-        f._fmt = metrics['format']
-        f._bsz = metrics['trace_bsize']
-        f._tracecount = metrics['trace_count']
-        f._ext_headers = (f._tr0 - 3600) // 3200  # should probably be from C
-
         dt = segyio.tools.dt(f, fallback_dt = 4000.0) / 1000.0
         t0 = f.header[0][segyio.TraceField.DelayRecordingTime]
-        samples = metrics['sample_count']
+        samples = metrics['samplecount']
         f._samples = (numpy.arange(samples, dtype = numpy.single) * dt) + t0
 
     except:
@@ -90,23 +90,18 @@ def open(filename, mode="r", iline = 189,
         return f
 
     try:
-        cube_metrics = segyio._segyio.init_cube_metrics(f.xfd,
-                                                        iline,
-                                                        xline,
-                                                        f.tracecount,
-                                                        f._tr0,
-                                                        f._bsz)
+        cube_metrics = f.xfd.cube_metrics(iline, xline)
         f._sorting   = cube_metrics['sorting']
         iline_count  = cube_metrics['iline_count']
         xline_count  = cube_metrics['xline_count']
         offset_count = cube_metrics['offset_count']
         metrics.update(cube_metrics)
 
-        line_metrics = segyio._segyio.init_line_metrics(f.sorting,
-                                                        f.tracecount,
-                                                        iline_count,
-                                                        xline_count,
-                                                        offset_count)
+        line_metrics = segyio._segyio.line_metrics(f.sorting,
+                                                   f.tracecount,
+                                                   iline_count,
+                                                   xline_count,
+                                                   offset_count)
 
         f._iline_length = line_metrics['iline_length']
         f._iline_stride = line_metrics['iline_stride']
@@ -117,7 +112,7 @@ def open(filename, mode="r", iline = 189,
         f._ilines  = numpy.zeros(iline_count,  dtype = numpy.intc)
         f._xlines  = numpy.zeros(xline_count,  dtype = numpy.intc)
         f._offsets = numpy.zeros(offset_count, dtype = numpy.intc)
-        segyio._segyio.init_indices(f.xfd, metrics, f.ilines, f.xlines, f.offsets)
+        f.xfd.indices(metrics, f.ilines, f.xlines, f.offsets)
 
         if numpy.unique(f.ilines).size != f.ilines.size:
             raise ValueError( "Inlines inconsistent - expect all inlines to be unique")
