@@ -15,6 +15,11 @@ segyio which you can find in the examples directory or where your distribution
 installs example programs.
 """
 import itertools
+try:
+    from future_builtins import zip
+    range = xrange
+except (NameError, ImportError): pass
+
 import numpy as np
 
 from segyio._header import Header
@@ -26,20 +31,12 @@ import segyio._segyio as _segyio
 
 from segyio.tracesortingformat import TraceSortingFormat
 
-try:
-    from itertools import izip as zip
-except ImportError:  # will be 3.x series
-    pass
-
-try: xrange
-except NameError: pass
-else: range = xrange
 
 class SegyFile(object):
 
     _unstructured_errmsg = "File opened in unstructured mode."
 
-    def __init__(self, filename, mode, iline=189, xline=193):
+    def __init__(self, filename, mode, iline=189, xline=193, binary=None):
         """
         Constructor, internal.
         """
@@ -52,10 +49,7 @@ class SegyFile(object):
         # property value holders
         self._ilines = None
         self._xlines = None
-        self._tracecount = None
-        self._sorting = None
         self._offsets = None
-        self._ext_headers = None
         self._samples = None
 
         # private values
@@ -63,9 +57,6 @@ class SegyFile(object):
         self._iline_stride = None
         self._xline_length = None
         self._xline_stride = None
-        self._fmt = None
-        self._tr0 = None
-        self._bsz = None
 
         self._trace = Trace(self)
         self._header = Header(self)
@@ -73,7 +64,11 @@ class SegyFile(object):
         self._xline = None
         self._gather = None
 
-        self.xfd = _segyio.open(filename, mode)
+        self.xfd = _segyio.segyiofd(filename, mode, binary)
+        metrics = self.xfd.metrics()
+        self._fmt = metrics['format']
+        self._tracecount = metrics['tracecount']
+        self._ext_headers = metrics['ext_headers']
 
         super(SegyFile, self).__init__()
 
@@ -134,7 +129,7 @@ class SegyFile(object):
                 ...     # write something to f
                 ...     f.flush()
         """
-        _segyio.flush(self.xfd)
+        self.xfd.flush()
 
     def close(self):
         """Close the file.
@@ -146,7 +141,7 @@ class SegyFile(object):
         It is not necessary to call this method if you're using the `with`
         statement, which will close the file for you.
         """
-        _segyio.close(self.xfd)
+        self.xfd.close()
 
     def mmap(self):
         """Memory map the file
@@ -176,7 +171,7 @@ class SegyFile(object):
             >>> # keep using segyio as per usual
             >>> print( f.trace[10] )
         """
-        return _segyio.mmap(self.xfd)
+        return self.xfd.mmap()
 
     @property
     def sorting(self):
@@ -190,7 +185,7 @@ class SegyFile(object):
 
     @property
     def samples(self):
-        """ :rtype: int """
+        """ :rtype: numpy.ndarray """
         return self._samples
 
     @property
@@ -380,9 +375,7 @@ class SegyFile(object):
                 traces = self.tracecount
                 start, stop, step = rng.indices(traces)
                 attrs = np.empty(len(range(*rng.indices(traces))), dtype = np.intc)
-                return _segyio.field_forall(self.xfd, attrs,
-                                            start, stop, step, field,
-                                            self._tr0, self._bsz)
+                return self.xfd.field_forall(attrs, start, stop, step, field)
 
             def _getitem_list(inner, xs):
                 if not isinstance(xs, np.ndarray):
@@ -390,8 +383,7 @@ class SegyFile(object):
 
                 xs = xs.astype(dtype = np.intc, order = 'C', copy = False)
                 attrs = np.empty(len(xs), dtype = np.intc)
-                return _segyio.field_foreach(self.xfd, attrs, xs, field,
-                                          self._tr0, self._bsz)
+                return self.xfd.field_foreach(attrs, xs, field)
 
         return attr()
 
@@ -522,11 +514,7 @@ class SegyFile(object):
 
     def _fread_line(self, trace0, length, stride, buf):
         offsets = len(self.offsets)
-        return _segyio.read_line(self.xfd, trace0,
-                                length, stride, offsets,
-                                buf,
-                                self._tr0, self._bsz,
-                                self._fmt, len(self.samples))
+        return self.xfd.getline(trace0, length, stride, offsets, buf)
 
     @property
     def ilines(self):
@@ -659,7 +647,7 @@ class SegyFile(object):
             for i, v in zip(range(t0, t0 + (step * length), step), val):
                 Trace.write_trace(i, v, self)
 
-        self._iline = Line(self, il_len, il_stride, lines, other_lines, buffn, readfn, writefn, "Inline")
+        self._iline = Line(self, il_len, il_stride, lines, other_lines, buffn, readfn, writefn, "inline")
         return self._iline
 
     @iline.setter
@@ -797,7 +785,7 @@ class SegyFile(object):
             for i, v in zip(range(t0, t0 + step * length, step), val):
                 Trace.write_trace(i, v, self)
 
-        self._xline = Line(self, xl_len, xl_stride, lines, other_lines, buffn, readfn, writefn, "Crossline")
+        self._xline = Line(self, xl_len, xl_stride, lines, other_lines, buffn, readfn, writefn, "crossline")
         return self._xline
 
     @xline.setter
@@ -860,10 +848,12 @@ class SegyFile(object):
 
         Since v1.1
 
-        This mode gives access to reading and writing functionality for depth slices.
-        The primary data type is the numpy ndarray. Depth slices can be accessed
-        individually or with python slices, and writing is done via assignment.
-        Note that each slice is returned as a numpy array, meaning
+        This mode gives access to reading and writing functionality for depth
+        slices, a horizontal cut of the survey.
+
+        The primary data type is the numpy ndarray. Depth slices can be
+        accessed individually or with python slices, and writing is done via
+        assignment.  Note that each slice is returned as a numpy array, meaning
         accessing the values of the slice is 0-indexed.
 
         Examples:
@@ -888,10 +878,10 @@ class SegyFile(object):
                 ...     print(np.average(depth_slice))
                 ...
 
-            Copy a line from file g to f::
+            Copy a slice from file g to f::
                 >>> f.depth_slice[4] = g.depth_slice[19]
 
-            Copy lines from the first line in g to f, starting at 10,
+            Copy slice from the first line in g to f, starting at 10,
             ending at 49 in f::
                 >>> f.depth_slice[10:50] = g.depth_slice
 
@@ -905,7 +895,7 @@ class SegyFile(object):
             destination file will be written.
 
             Copy depth slices from file f to file g::
-                >>> f.depth_slice = g.depth_slice.
+                >>> f.depth_slice = g.depth_slice
 
             Copy first half of the depth slices from g to f::
                 >>> f.depth_slice = g.depth_slice[:g.samples/2]]
@@ -923,18 +913,9 @@ class SegyFile(object):
 
         slice_trace_count = self._iline_length * self._xline_length
         offsets = len(self.offsets)
-        tr0 = self._tr0
-        bsz = self._bsz
-        fmt = self._fmt
 
         def readfn(depth, length, stride, buf):
-            _segyio.depth_slice(self.xfd, depth,
-                                slice_trace_count, offsets,
-                                buf,
-                                tr0, bsz,
-                                fmt,
-                                len(self.samples))
-            return buf
+            return self.xfd.getdepth(depth, slice_trace_count, offsets, buf)
 
         def writefn(depth, length, stride, val):
             val = buffn(val)
@@ -945,7 +926,7 @@ class SegyFile(object):
                 trace_buf[depth] = buf_view[i]
                 self.trace[i] = trace_buf
 
-        return Line(self, len(self.samples), 1, indices, other_indices, buffn, readfn, writefn, "Depth")
+        return Line(self, len(self.samples), 1, indices, other_indices, buffn, readfn, writefn, "depth")
 
     @depth_slice.setter
     def depth_slice(self, value):
@@ -1017,6 +998,9 @@ class SegyFile(object):
             Write a new textual header::
                 >>> f.text[0] = make_new_header()
 
+            Copy a tectual header::
+                >>> f.text[1] = g.text[0]
+
             Print a textual header line-by-line::
                 >>> # using zip, from the zip documentation
                 >>> text = f.text[0]
@@ -1025,28 +1009,7 @@ class SegyFile(object):
                 ...     print(line)
                 ...
         """
-
-        class TextHeader:
-
-            def __getitem__(inner, index):
-                if index > self.ext_headers:
-                    raise IndexError("Textual header %d not in file" % index)
-
-                return _segyio.read_textheader(self.xfd, index)
-
-            def __setitem__(inner, index, val):
-                if index > self.ext_headers:
-                    raise IndexError("Textual header %d not in file" % index)
-
-                _segyio.write_textheader(self.xfd, index, val)
-
-            def __repr__(inner):
-                return "Text(external_headers = {})".format(self.ext_headers)
-
-            def __str__(inner):
-                return '\n'.join(map(''.join, zip(*[iter(inner[0])] * 80)))
-
-        return TextHeader()
+        return TextHeader(self)
 
     @property
     def bin(self):
@@ -1144,8 +1107,33 @@ class spec:
         self.xlines = None
         self.offsets = [1]
         self.samples = None
-        self.tracecount = None
         self.ext_headers = 0
         self.format = None
         self.sorting = None
-        self.t0 = 1111.0
+
+class TextHeader(object):
+
+    def __init__(self, outer):
+        self.outer = outer
+
+    def __getitem__(self, index):
+        if not 0 <= index <= self.outer.ext_headers:
+            raise IndexError("Textual header {} not in file".format(index))
+
+        return self.outer.xfd.gettext(index)
+
+    def __setitem__(self, index, val):
+        if isinstance(val, TextHeader):
+            self[index] = val[0]
+            return
+
+        if not 0 <= index <= self.outer.ext_headers:
+            raise IndexError("Textual header {} not in file".format(index))
+
+        self.outer.xfd.puttext(index, val)
+
+    def __repr__(self):
+        return "Text(external_headers = {})".format(self.outer.ext_headers)
+
+    def __str__(self):
+        return '\n'.join(map(''.join, zip(*[iter(str(self[0]))] * 80)))
