@@ -14,7 +14,7 @@ class TestContext(object):
         self._cleanup = cleanup
 
     def __enter__(self):
-        temp_path = tempfile.mkdtemp("_%s" % self._name)
+        temp_path = tempfile.mkdtemp("_{}".format(self._name))
         os.chdir(temp_path)
         self._temp_path = os.getcwd()
         return self
@@ -28,7 +28,7 @@ class TestContext(object):
         if not os.path.isabs(source):
             source = os.path.join(self._cwd, source)
 
-        source_dir, source_filename = os.path.split(source)
+        _ , source_filename = os.path.split(source)
 
         target = os.path.join(self._temp_path, target, source_filename)
         try:
