@@ -2,7 +2,6 @@ from __future__ import absolute_import
 from __future__ import division
 import unittest
 
-import math
 from segyio import BinField
 from segyio import TraceField
 import numpy as np
@@ -129,5 +128,78 @@ class ToolsTest(unittest.TestCase):
                 close(right, rotation(f, line = 'slow'),  places = 3)
                 close(right, rotation(f, line = 'xline'), places = 3)
 
+    def test_metadata(self):
+        spec = segyio.spec()
+        spec.ilines = [1, 2, 3, 4, 5]
+        spec.xlines = [20, 21, 22, 23, 24]
+        spec.samples = list(range(0,200,4))
+        spec.sorting = 2
+        spec.format = 1
+
+        smallspec = segyio.tools.metadata(self.filename)
+
+        self.assertTrue(np.array_equal(spec.ilines, smallspec.ilines))
+        self.assertTrue(np.array_equal(spec.xlines, smallspec.xlines))
+        self.assertTrue(np.array_equal(spec.offsets, smallspec.offsets))
+        self.assertTrue(np.array_equal(spec.samples, smallspec.samples))
+        self.assertEqual(spec.sorting, smallspec.sorting)
+        self.assertEqual(spec.format, int(smallspec.format))
+
+    def test_resample(self):
+        with TestContext('resample-none') as context:
+            context.copy_file(self.filename)
+
+            old = list(range(0,200,4))
+
+            with segyio.open('small.sgy', 'r+') as f:
+                self.assertTrue(np.array_equal(old, f.samples))
+                segyio.tools.resample(f) # essentially a no-op
+                self.assertTrue(np.array_equal(old, f.samples))
+
+            with segyio.open('small.sgy') as f:
+                self.assertTrue(np.array_equal(old, f.samples))
+
+        with TestContext('resample-all') as context:
+            context.copy_file(self.filename)
+
+            old = list(range(0,200,4))
+            new = list(range(12,112,2))
+
+            with segyio.open('small.sgy', 'r+') as f:
+                self.assertTrue(np.array_equal(old, f.samples))
+                segyio.tools.resample(f, rate = 2, delay = 12)
+                self.assertTrue(np.array_equal(new, f.samples))
+
+            with segyio.open('small.sgy') as f:
+                self.assertTrue(np.array_equal(new, f.samples))
+
+        with TestContext('resample-rate') as context:
+            context.copy_file(self.filename)
+
+            old = list(range(0,200,4))
+            new = list(range(12,212,4))
+
+            with segyio.open('small.sgy', 'r+') as f:
+                self.assertTrue(np.array_equal(old, f.samples))
+                segyio.tools.resample(f, delay = 12)
+                self.assertTrue(np.array_equal(new, f.samples))
+
+            with segyio.open('small.sgy') as f:
+                self.assertTrue(np.array_equal(new, f.samples))
+
+        with TestContext('resample-delay') as context:
+            context.copy_file(self.filename)
+
+            old = list(range(0,200,4))
+            new = list(range(0,100,2))
+
+            with segyio.open('small.sgy', 'r+') as f:
+                self.assertTrue(np.array_equal(old, f.samples))
+                segyio.tools.resample(f, rate = 2000, micro = True)
+                self.assertTrue(np.array_equal(new, f.samples))
+
+            with segyio.open('small.sgy') as f:
+                self.assertTrue(np.array_equal(new, f.samples))
+
 if __name__ == '__main__':
     unittest.main()
